---
network_config:
- type: interface
  name: nic1
  use_dhcp: false
  addresses:
  - ip_netmask: {{ ctlplane_ip }}/{{ ctlplane_subnet_cidr }}
- type: interface
  name: nic2
  use_dhcp: false
- type: vlan
  device: nic2
  vlan_id: {{ internal_api_vlan_id }}
  addresses:
  - ip_netmask: {{ internal_api_ip }}/{{ internal_api_cidr }}
- type: ovs_bridge
  name: bridge_name
  dns_servers: {{ ctlplane_dns_nameservers }}
  members:
  - type: interface
    name: nic3
    primary: true
  - type: vlan
    vlan_id: {{ tenant_vlan_id }}
    addresses:
    - ip_netmask: {{ tenant_ip }}/{{ tenant_cidr }}
